Attribute VB_Name = "PicBlt"
  ' All variables MUST be declared
  Option Explicit

  ' Win32 API Declarations, Structures, and Constants
  Private Declare Function BitBlt Lib "gdi32" (ByVal hDestDC As Long, _
    ByVal X As Long, ByVal Y As Long, ByVal nWidth As Long, ByVal nHeight As Long, _
    ByVal hSrcDC As Long, ByVal xsrc As Long, ByVal ysrc As Long, ByVal dwRop As Long) _
    As Long
  Private Declare Function SetBkColor Lib "gdi32" (ByVal hdc As Long, _
    ByVal crColor As Long) As Long
  Private Declare Function CreateCompatibleDC Lib "gdi32" (ByVal hdc As Long) As Long
  Private Declare Function DeleteDC Lib "gdi32" (ByVal hdc As Long) As Long
  Private Declare Function CreateBitmap Lib "gdi32" (ByVal nWidth As Long, _
    ByVal nHeight As Long, ByVal nPlanes As Long, ByVal nBitCount As Long, _
    lpBits As Any) As Long
  Private Declare Function CreateCompatibleBitmap Lib "gdi32" (ByVal hdc As Long, _
    ByVal nWidth As Long, ByVal nHeight As Long) As Long
  Private Declare Function GetObj Lib "gdi32" Alias "GetObjectA" (ByVal hObject As Long, _
    ByVal nCount As Long, lpObject As Any) As Long
  Private Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, _
    ByVal hObject As Long) As Long
  Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
  Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

  ' Ternary raster operations
  Private Const SRCCOPY = &HCC0020         ' (DWORD) dest = source
  Private Const SRCPAINT = &HEE0086        ' (DWORD) dest = source OR dest
  Private Const SRCAND = &H8800C6          ' (DWORD) dest = source AND dest
  Private Const NOTSRCCOPY = &H330008      ' (DWORD) dest = (NOT source)

Public Sub pic_blt(ByVal bltType As Integer, _
  ByVal fadeType As Integer, _
  ByRef frmSource As Form, _
  ByRef picSource As PictureBox, _
  ByRef picDest As PictureBox, _
  ByVal sec As Single, _
  Optional ByVal transcolor As Long)
                    
  '*************************************************************************
  '  Sub fade_blt
  '*************************************************************************
  ' 32-bit transparent fade-in blt
  
  ' Parameters
  ' bltType:    = 0 regular blt, = 1 transparent blt
  ' fadeType:   = 0 fade in, = 1 fade out
  ' frmSource:  Source form on which animation occurs
  ' picSource:  Source picturebox containing bitmap picture
  ' picDest:    Destination picturebox
  ' sec:        The elapsed time of the fade-in animation
  ' transcolor: Color to use as transparent color
   
  Dim OrigColor As Long     ' Holds original background color
  Dim xDest As Long         ' location value in pixels of destination picturebox (x)
  Dim yDest As Long         ' location value in pixels of destination picturebox (y)
  Dim X As Long             ' location value in pixels of source pixel blt box
  Dim Y As Long             ' location value in pixels of source pixel blt box
  Dim nWidth As Long        ' width of destination bitmap
  Dim nHeight As Long       ' height of destimation bitmap
  Dim saveDC As Long        ' Backup copy of source bitmap
  Dim maskDC As Long        ' Mask bitmap (monochrome)
  Dim invDC As Long         ' Inverse of mask bitmap (monochrome)
  Dim resultDC As Long      ' Combination of source bitmap & background
  Dim origDC As Long        ' DC for original place cuttout on form before blitting
      
  Dim hSaveBmp As Long      ' Bitmap stores backup copy of source bitmap
  Dim hMaskBmp As Long      ' Bitmap stores mask (monochrome)
  Dim hInvBmp As Long       ' Bitmap holds inverse of mask (monochrome)
  Dim hResultBmp As Long    ' Bitmap combination of source & background
  Dim hOrigBmp As Long      ' Bitmap that stores the form cuttout bitmap
  Dim hSavePrevBmp As Long  ' Holds previous bitmap in saved DC
  Dim hMaskPrevBmp As Long  ' Holds previous bitmap in the mask DC
  Dim hInvPrevBmp As Long   ' Holds previous bitmap in inverted mask DC
  Dim hDestPrevBmp As Long  ' Holds previous bitmap in destination DC
  Dim hOrigPrevBmp As Long  ' Holds previous bitmap in original DC
   
  Dim Start As Single
  Dim i, J As Integer
   
  ' set up properties for the animation objects
  ' NOTE: the following permanently
  ' changes the properties of the form and picbox objects.  You may need to insert
  ' code at the end to set the properties back to what they were originally
  With frmSource
    .ScaleMode = vbPixels
    .AutoRedraw = False
  End With
   
  With picSource
    .ScaleMode = vbPixels
    .AutoRedraw = True
  End With
   
  With picDest
    .ScaleMode = vbPixels
    .AutoRedraw = False
    nWidth = .ScaleWidth
    nHeight = .ScaleHeight
    xDest = .Left
    yDest = .Top
  End With
  '============================================================
  ' Create the DC's and bitmaps for regular and transparent blt
  '============================================================
  
  ' Create Device Handles (DC's)
  resultDC = CreateCompatibleDC(picDest.hdc)
  origDC = CreateCompatibleDC(picDest.hdc)
  
  ' Creat color bitmaps
  hResultBmp = CreateCompatibleBitmap(picSource.hdc, nWidth, nHeight)
  hOrigBmp = CreateCompatibleBitmap(picSource.hdc, nWidth, nHeight)
  
  ' select in objects
  hDestPrevBmp = SelectObject(resultDC, hResultBmp)
  hOrigPrevBmp = SelectObject(origDC, hOrigBmp)
  
  '===========================================================
  ' Create DC's and bitmaps for transparent blt only
  '===========================================================
  If bltType = 1 Then
    saveDC = CreateCompatibleDC(picDest.hdc)
    maskDC = CreateCompatibleDC(picDest.hdc)
    invDC = CreateCompatibleDC(picDest.hdc)
    
    ' Create monochrome bitmaps for the mask-related bitmaps
    hMaskBmp = CreateBitmap(nWidth, nHeight, 1, 1, ByVal 0&)
    hInvBmp = CreateBitmap(nWidth, nHeight, 1, 1, ByVal 0&)
     
    ' Create a color bitmap for intermediate transparent blt
    hSaveBmp = CreateCompatibleBitmap(picSource.hdc, nWidth, nHeight)
  
    ' Select bitmaps into DCs.
    hSavePrevBmp = SelectObject(saveDC, hSaveBmp)
    hMaskPrevBmp = SelectObject(maskDC, hMaskBmp)
    hInvPrevBmp = SelectObject(invDC, hInvBmp)
    
    ' ==================================================================
    ' create transparent bitmap
    ' ==================================================================
  
    ' Create mask: set background color of source to transparent color
    OrigColor = SetBkColor(picSource.hdc, transcolor)
    Call BitBlt(maskDC, 0, 0, nWidth, nHeight, picSource.hdc, 0, 0, vbSrcCopy)
    transcolor = SetBkColor(picSource.hdc, OrigColor)
  
    ' Create inverse of mask to AND w/ source & combine w/ background
    Call BitBlt(invDC, 0, 0, nWidth, nHeight, maskDC, 0, 0, vbNotSrcCopy)
  
    ' Copy background bitmap to result and original
    picDest.Visible = False
    frmSource.Refresh
    Call BitBlt(resultDC, 0, 0, nWidth, nHeight, frmSource.hdc, xDest, yDest, vbSrcCopy)
    Call BitBlt(origDC, 0, 0, nWidth, nHeight, frmSource.hdc, xDest, yDest, vbSrcCopy)
  
    ' AND mask bitmap w/ result DC to punch hole in the background by
    ' painting black area for non-transparent portion of source bitmap
    Call BitBlt(resultDC, 0, 0, nWidth, nHeight, maskDC, 0, 0, vbSrcAnd)
  
    ' get overlapper
    Call BitBlt(saveDC, 0, 0, nWidth, nHeight, picSource.hdc, 0, 0, vbSrcCopy)
  
    ' AND with inverse monochrome mask
    Call BitBlt(saveDC, 0, 0, nWidth, nHeight, invDC, 0, 0, vbSrcAnd)
  
    ' XOR these two
    Call BitBlt(resultDC, 0, 0, nWidth, nHeight, saveDC, 0, 0, vbSrcInvert)
    
    ' blt in the whole background into the transparent picturebox
    picDest.Visible = True
    picDest.Refresh
    
    Call BitBlt(picDest.hdc, 0, 0, nWidth, nHeight, origDC, 0, 0, vbSrcCopy)
    
    
  End If
  
  '=====================================================================
  ' do the fade in
  '=====================================================================
  frmSplash.picLogo.Visible = True
 
  Start = Timer                           ' Set start time
  Do While Timer < Start + sec            ' Do an n second fade loop
    ' get a random pixel start point X,Y
    X = Int(nWidth / 2 * Rnd)
    Y = Int(nHeight / 2 * Rnd)
  
    ' Fade-In: Display pieces of the transparent bitmap into picturebox
    Call BitBlt(picDest.hdc, X * 2, Y * 2, 2, 2, resultDC, X * 2, Y * 2, vbSrcCopy)
  Loop
  
  ' blt the final bitmap in to the whole pic box to display the whole thing
  Call BitBlt(picDest.hdc, 0, 0, nWidth, nHeight, resultDC, 0, 0, vbSrcCopy)
    
  '==========================================================================
  ' Cleanup by deallocating memory resources
  '==========================================================================
  Call SelectObject(resultDC, hDestPrevBmp)
  Call SelectObject(origDC, hOrigPrevBmp)
  Call DeleteObject(hResultBmp)
  Call DeleteObject(hOrigBmp)
  Call DeleteDC(resultDC)
  Call DeleteDC(origDC)
  
End Sub
